#include "adjdirect.h"
#include "gpav.h"
/* An example of using the GPAV algorithm


	Code written by O. Sysoev

     Reference: O. Burdakov, O. Sysoev, A. Grimvall and M. Hussian. 
                "An O(n2) algorithm for isotonic regression."
                 In: G. Di Pillo and M. Roma (Eds) Large-Scale Nonlinear Optimization.
                 Series: Nonconvex Optimization and Its Applications,
                 Springer-Verlag, (2006) 83, pp. 25-33
*/
int main(){
	double** X;
	double* Y;
	/*specify amount of observations*/
	int N=20000;

	/*specify dimensionality p: x \in R^p*/
	int p=3;
	/*uploading data, note: data needs to be ordered with respect to X in advance*/
	upload_data("Simulation2.csv",X,Y,N,p);
	int k=1;
	double* Ynew=nullptr;
	vector<int> rord;

	/*construct adjacency matrix from data*/
	vector<pair<int,int>>* adjR=adj_direct(X,Y,N,p, rord,Ynew);
	vector<pair<int,int>> adj=*adjR;
	
	/*running the GPAV on the reordered data*/
	double* Yfit=GPAV(adj, N,Ynew);

	double* Yfit1=new double[N];

	double objective=0;
	/*computing fitted value for the original order in the data*/
	for(int i=0; i<N; i++) Yfit1[rord[i]]=Yfit[i];

	for(int i=0; i<N; i++) objective+=(Y[i]-Yfit1[i])*(Y[i]-Yfit1[i]);

	std::cout<< "The objective function value is: "<<objective;

	delete[] Y;
	delete[] Yfit;
	delete[] Ynew;
	delete[] Yfit1;
	for(int i=0;i<p; i++) delete[] X[i];
	delete[] X;

}